library angular2.src.core.compiler.dynamic_component_loader;

import "package:angular2/di.dart"
    show Key, Injector, Injectable, ResolvedBinding;
import "compiler.dart" show Compiler;
import "directive_metadata_reader.dart" show DirectiveMetadataReader;
import "package:angular2/src/facade/lang.dart"
    show Type, BaseException, stringify, isPresent;
import "package:angular2/src/facade/collection.dart" show List;
import "package:angular2/src/facade/async.dart" show Future;
import "package:angular2/src/core/annotations/annotations.dart" show Component;
import "package:angular2/src/core/compiler/view_factory.dart" show ViewFactory;
import "package:angular2/src/render/api.dart" show Renderer;
import "element_injector.dart"
    show
        ElementRef,
        DirectiveRef,
        ComponentRef; /**
 * Service for dynamically loading a Component into an arbitrary position in the internal Angular
 * application tree.
 */

@Injectable()
class DynamicComponentLoader {
  Compiler _compiler;
  ViewFactory _viewFactory;
  Renderer _renderer;
  DirectiveMetadataReader _directiveMetadataReader;
  DynamicComponentLoader(Compiler compiler,
      DirectiveMetadataReader directiveMetadataReader, Renderer renderer,
      ViewFactory viewFactory) {
    this._compiler = compiler;
    this._directiveMetadataReader = directiveMetadataReader;
    this._renderer = renderer;
    this._viewFactory = viewFactory;
  } /**
   * Loads a component into the location given by the provided ElementRef. The loaded component
   * receives injection as if it in the place of the provided ElementRef.
   */
  Future<ComponentRef> loadIntoExistingLocation(Type type, ElementRef location,
      [Injector injector = null]) {
    this._assertTypeIsComponent(type);
    var directiveMetadata = this._directiveMetadataReader.read(type);
    var inj = this._componentAppInjector(
        location, injector, directiveMetadata.resolvedInjectables);
    var hostEi = location.elementInjector;
    var hostView = location.hostView;
    return this._compiler.compile(type).then((componentProtoView) {
      var context = hostEi.dynamicallyCreateComponent(
          type, directiveMetadata.annotation, inj);
      var componentView = this._instantiateAndHydrateView(componentProtoView,
          injector, hostEi,
          context); //TODO(vsavkin): do not use component child views as we need to clear the dynamically created views
      //same problem exists on the render side
      hostView.addComponentChildView(componentView);
      this._renderer.setDynamicComponentView(hostView.render,
          location.boundElementIndex,
          componentView.render); // TODO(vsavkin): return a component ref that dehydrates the component view and removes it
      // from the component child views
      return new ComponentRef(Key.get(type), hostEi, componentView);
    });
  } /**
   * Loads a component as a child of the View given by the provided ElementRef. The loaded
   * component receives injection normally as a hosted view.
   *
   * TODO(vsavkin, jelbourn): remove protoViewFactory after render layer exists.
   */
  Future<ComponentRef> loadIntoNewLocation(
      dynamic elementOrSelector, Type type, ElementRef location,
      [Injector injector = null]) {
    this._assertTypeIsComponent(type);
    var inj = this._componentAppInjector(location, injector,
        null); //TODO(tbosch) this should always be a selector
    return this._compiler.compileRoot(elementOrSelector, type).then((pv) {
      var hostView = this._instantiateAndHydrateView(pv, inj, null,
          new Object()); // TODO(vsavkin): return a component ref that dehydrates the host view
      return new ComponentRef(Key.get(type), hostView.elementInjectors[0],
          hostView.componentChildViews[0]);
    });
  }
  _componentAppInjector(
      location, Injector injector, List<ResolvedBinding> resolvedBindings) {
    var inj = isPresent(injector) ? injector : location.injector;
    return isPresent(resolvedBindings)
        ? inj.createChildFromResolved(resolvedBindings)
        : inj;
  }
  _instantiateAndHydrateView(
      protoView, injector, hostElementInjector, context) {
    var componentView = this._viewFactory.getView(protoView);
    componentView.hydrate(injector, hostElementInjector, context, null);
    return componentView;
  } /** Asserts that the type being dynamically instantiated is a Component. */
  _assertTypeIsComponent(Type type) {
    var annotation = this._directiveMetadataReader.read(type).annotation;
    if (!(annotation is Component)) {
      throw new BaseException(
          '''Could not load \'${ stringify ( type )}\' because it is not a component.''');
    }
  }
}
