library angular2.src.di.annotations;

import "package:angular2/src/facade/lang.dart"
    show
        CONST; /**
 * A parameter annotation that creates a synchronous eager dependency.
 *
 * ```
 * class AComponent {
 *   constructor(@Inject('aServiceToken') aService) {}
 * }
 * ```
 *
 */

class Inject {
  final token;
  @CONST() const Inject(token) : token = token;
} /**
 * A parameter annotation that creates an asynchronous eager dependency.
 *
 * ```
 * class AComponent {
 *   constructor(@InjectPromise('aServiceToken') aServicePromise) {
 *     aServicePromise.then(aService => ...);
 *   }
 * }
 * ```
 *
 */
class InjectPromise {
  final token;
  @CONST() const InjectPromise(token) : token = token;
} /**
 * A parameter annotation that creates a synchronous lazy dependency.
 *
 * ```
 * class AComponent {
 *   constructor(@InjectLazy('aServiceToken') aServiceFn) {
 *     aService = aServiceFn();
 *   }
 * }
 * ```
 *
 */
class InjectLazy {
  final token;
  @CONST() const InjectLazy(token) : token = token;
} /**
 * A parameter annotation that marks a dependency as optional.
 *
 * ```
 * class AComponent {
 *   constructor(@Optional() dp:Dependency) {
 *     this.dp = dp;
 *   }
 * }
 * ```
 *
 */
class Optional {
  @CONST() const Optional();
} /**
 * `DependencyAnnotation` is used by the framework to extend DI.
 *
 * Only annotations implementing `DependencyAnnotation` will be added
 * to the list of dependency properties.
 *
 * For example:
 *
 * ```
 * class Parent extends DependencyAnnotation {}
 * class NotDependencyProperty {}
 *
 * class AComponent {
 *   constructor(@Parent @NotDependencyProperty aService:AService) {}
 * }
 * ```
 *
 * will create the following dependency:
 *
 * ```
 * new Dependency(Key.get(AService), [new Parent()])
 * ```
 *
 * The framework can use `new Parent()` to handle the `aService` dependency
 * in a specific way.
 *
 */
class DependencyAnnotation {
  @CONST() const DependencyAnnotation();
  get token {
    return null;
  }
} /**
 * A class annotation that marks a class as available to `Injector`s for
 * creation.
 *
 * ```
 * class NeedsService {
 *   constructor(svc:UsefulService) {}
 * }
 *
 * @Injectable
 * class UsefulService {}
 * ```
 */
class Injectable {
  @CONST() const Injectable();
}
