library angular2.src.di.binding;

import "package:angular2/src/facade/lang.dart"
    show Type, isBlank, isPresent, CONST;
import "package:angular2/src/facade/collection.dart"
    show List, MapWrapper, ListWrapper;
import "package:angular2/src/reflection/reflection.dart" show reflector;
import "key.dart" show Key;
import "annotations.dart"
    show Inject, InjectLazy, InjectPromise, Optional, DependencyAnnotation;
import "exceptions.dart" show NoAnnotationError;

class Dependency {
  Key key;
  bool asPromise;
  bool lazy;
  bool optional;
  List properties;
  Dependency(
      Key key, bool asPromise, bool lazy, bool optional, List properties) {
    this.key = key;
    this.asPromise = asPromise;
    this.lazy = lazy;
    this.optional = optional;
    this.properties = properties;
  }
  static fromKey(Key key) {
    return new Dependency(key, false, false, false, []);
  }
}
var _EMPTY_LIST = []; /**
 * Declaration of a dependency binding.
 */
class Binding {
  final token;
  final Type toClass;
  final toValue;
  final toAlias;
  final Function toFactory;
  final Function toAsyncFactory;
  final List dependencies;
  @CONST() const Binding(token,
      {toClass, toValue, toAlias, toFactory, toAsyncFactory, deps})
      : token = token,
        toClass = toClass,
        toValue = toValue,
        toAlias = toAlias,
        toFactory = toFactory,
        toAsyncFactory = toAsyncFactory,
        dependencies = deps;
  ResolvedBinding resolve() {
    Function factoryFn;
    var resolvedDeps;
    var isAsync = false;
    if (isPresent(this.toClass)) {
      factoryFn = reflector.factory(this.toClass);
      resolvedDeps = _dependenciesFor(this.toClass);
    } else if (isPresent(this.toAlias)) {
      factoryFn = (aliasInstance) => aliasInstance;
      resolvedDeps = [Dependency.fromKey(Key.get(this.toAlias))];
    } else if (isPresent(this.toFactory)) {
      factoryFn = this.toFactory;
      resolvedDeps = _constructDependencies(this.toFactory, this.dependencies);
    } else if (isPresent(this.toAsyncFactory)) {
      factoryFn = this.toAsyncFactory;
      resolvedDeps =
          _constructDependencies(this.toAsyncFactory, this.dependencies);
      isAsync = true;
    } else {
      factoryFn = () => this.toValue;
      resolvedDeps = _EMPTY_LIST;
    }
    return new ResolvedBinding(
        Key.get(this.token), factoryFn, resolvedDeps, isAsync);
  }
} /// Dependency binding with resolved keys and dependencies.
class ResolvedBinding {
  Key key;
  Function factory;
  List<Dependency> dependencies;
  bool providedAsPromise;
  ResolvedBinding(Key key, Function factory, List<Dependency> dependencies,
      bool providedAsPromise) {
    this.key = key;
    this.factory = factory;
    this.dependencies = dependencies;
    this.providedAsPromise = providedAsPromise;
  }
} /**
 * Provides fluent API for imperative construction of [Binding] objects.
 */
BindingBuilder bind(token) {
  return new BindingBuilder(token);
} /**
 * Helper class for [bind] function.
 */
class BindingBuilder {
  var token;
  BindingBuilder(token) {
    this.token = token;
  }
  Binding toClass(Type type) {
    return new Binding(this.token, toClass: type);
  }
  Binding toValue(value) {
    return new Binding(this.token, toValue: value);
  }
  Binding toAlias(aliasToken) {
    return new Binding(this.token, toAlias: aliasToken);
  }
  Binding toFactory(Function factoryFunction, [List dependencies = null]) {
    return new Binding(this.token,
        toFactory: factoryFunction, deps: dependencies);
  }
  Binding toAsyncFactory(Function factoryFunction, [List dependencies = null]) {
    return new Binding(this.token,
        toAsyncFactory: factoryFunction, deps: dependencies);
  }
}
_constructDependencies(Function factoryFunction, List dependencies) {
  return isBlank(dependencies)
      ? _dependenciesFor(factoryFunction)
      : ListWrapper.map(dependencies, (t) => Dependency.fromKey(Key.get(t)));
}
List _dependenciesFor(typeOrFunc) {
  var params = reflector.parameters(typeOrFunc);
  if (isBlank(params)) return [];
  if (ListWrapper.any(
      params, (p) => isBlank(p))) throw new NoAnnotationError(typeOrFunc);
  return ListWrapper.map(params, (p) => _extractToken(typeOrFunc, p));
}
_extractToken(typeOrFunc, annotations) {
  var depProps = [];
  var token = null;
  var optional = false;
  var lazy = false;
  var asPromise = false;
  for (var i = 0; i < annotations.length; ++i) {
    var paramAnnotation = annotations[i];
    if (paramAnnotation is Type) {
      token = paramAnnotation;
    } else if (paramAnnotation is Inject) {
      token = paramAnnotation.token;
    } else if (paramAnnotation is InjectPromise) {
      token = paramAnnotation.token;
      asPromise = true;
    } else if (paramAnnotation is InjectLazy) {
      token = paramAnnotation.token;
      lazy = true;
    } else if (paramAnnotation is Optional) {
      optional = true;
    } else if (paramAnnotation is DependencyAnnotation) {
      if (isPresent(paramAnnotation.token)) {
        token = paramAnnotation.token;
      }
      ListWrapper.push(depProps, paramAnnotation);
    }
  }
  if (isPresent(token)) {
    return _createDependency(token, asPromise, lazy, optional, depProps);
  } else {
    throw new NoAnnotationError(typeOrFunc);
  }
}
Dependency _createDependency(token, asPromise, lazy, optional, depProps) {
  return new Dependency(Key.get(token), asPromise, lazy, optional, depProps);
}
